@@ -14,21 +14,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from oslo.config import cfg
-import logging
+import traceback
 
-from quantumclient import client as q_client
-from quantumclient.common import exceptions as q_exc
-from quantumclient.v2_0 import client as q_clientv2
+try:
+    from neutronclient import client as q_client
+    from neutronclient.common import exceptions as q_exc
+    from neutronclient.common.exceptions import (NeutronClientException as
+                                                 client_exc)
+    from neutronclient.v2_0 import client as q_clientv2
+except ImportError:
+    from quantumclient import client as q_client
+    from quantumclient.common import exceptions as q_exc
+    from quantumclient.common.exceptions import (QuantumClientException as
+                                                 client_exc)
+    from quantumclient.v2_0 import client as q_clientv2
 
 from ryu.app import conf_switch_key as cs_key
 from ryu.app import rest_nw_id
 from ryu.base import app_manager
 from ryu.controller import (conf_switch,
                             dpset,
-                            event,
                             handler,
                             network)
+from ryu import exception as ryu_exc
 from ryu.lib import dpid as dpid_lib
 from ryu.lib import mac as mac_lib
 from ryu.lib import quantum_ifaces
@@ -36,17 +44,14 @@ from ryu.lib.ovs import bridge
 from ryu.lib.quantum_ifaces import QuantumIfaces
 
 
-CONF = cfg.CONF
-
-
-def _get_auth_token(logger):
+def _get_auth_token(CONF, logger):
     httpclient = q_client.HTTPClient(
-        username=CONF.quantum_admin_username,
-        tenant_name=CONF.quantum_admin_tenant_name,
-        password=CONF.quantum_admin_password,
-        auth_url=CONF.quantum_admin_auth_url,
-        timeout=CONF.quantum_url_timeout,
-        auth_strategy=CONF.quantum_auth_strategy)
+        username=CONF.neutron_admin_username,
+        tenant_name=CONF.neutron_admin_tenant_name,
+        password=CONF.neutron_admin_password,
+        auth_url=CONF.neutron_admin_auth_url,
+        timeout=CONF.neutron_url_timeout,
+        auth_strategy=CONF.neutron_auth_strategy)
     try:
         httpclient.authenticate()
     except (q_exc.Unauthorized, q_exc.Forbidden, q_exc.EndpointNotFound) as e:
@@ -56,15 +61,15 @@ def _get_auth_token(logger):
     return httpclient.auth_token
 
 
-def _get_quantum_client(token):
+def _get_quantum_client(CONF, token):
     if token:
         my_client = q_clientv2.Client(
-            endpoint_url=CONF.quantum_url,
-            token=token, timeout=CONF.quantum_url_timeout)
+            endpoint_url=CONF.neutron_url,
+            token=token, timeout=CONF.neutron_url_timeout)
     else:
         my_client = q_clientv2.Client(
-            endpoint_url=CONF.quantum_url,
-            auth_strategy=None, timeout=CONF.quantum_url_timeout)
+            endpoint_url=CONF.neutron_url,
+            auth_strategy=None, timeout=CONF.neutron_url_timeout)
     return my_client
 
 
@@ -121,19 +126,17 @@ class OVSPort(object):
 
 
 class OVSSwitch(object):
-    def __init__(self, dpid, nw, ifaces, logger):
+    def __init__(self, CONF, dpid, nw, ifaces, logger):
         # TODO: clean up
-        token = None
-        if CONF.quantum_auth_strategy:
-            token = _get_auth_token(logger)
-        q_api = _get_quantum_client(token)
-
+        self.CONF = CONF
         self.dpid = dpid
         self.network_api = nw
         self.ifaces = ifaces
         self.logger = logger
-        self.q_api = q_api
-        self.ctrl_addr = CONF.quantum_controller_addr
+        self._q_api = None      # lazy initialization
+        self.ctrl_addr = self.CONF.neutron_controller_addr
+        if not self.ctrl_addr:
+            raise ValueError('option neutron_controler_addr must be speicfied')
 
         self.ovsdb_addr = None
         self.tunnel_ip = None
@@ -143,6 +146,15 @@ class OVSSwitch(object):
 
         super(OVSSwitch, self).__init__()
 
+    @property
+    def q_api(self):
+        if self._q_api is None:
+            token = None
+            if self.CONF.neutron_auth_strategy:
+                token = _get_auth_token(self.CONF, self.logger)
+            self._q_api = _get_quantum_client(self.CONF, token)
+        return self._q_api
+
     def set_ovsdb_addr(self, dpid, ovsdb_addr):
         # easy check if the address format valid
         self.logger.debug('set_ovsdb_addr dpid %s ovsdb_addr %s',
@@ -161,7 +173,7 @@ class OVSSwitch(object):
         self.ovsdb_addr = ovsdb_addr
         if self.ovs_bridge is None:
             self.logger.debug('ovsdb: adding ports %s', self.ports)
-            ovs_bridge = bridge.OVSBridge(dpid, ovsdb_addr)
+            ovs_bridge = bridge.OVSBridge(self.CONF, dpid, ovsdb_addr)
             self.ovs_bridge = ovs_bridge
             ovs_bridge.init()
             # TODO: for multi-controller
@@ -180,6 +192,11 @@ class OVSSwitch(object):
                                          self.dpid, port.ofport)
 
     def _update_vif_port(self, port, add=True):
+        # When ovs port is updated, the corresponding network id may or
+        # may not exist because the order between the notification of
+        # ovs port deletion via OVSDB protocol and the notification
+        # network id/port deletion via REST from quantum plugin
+        # isn't deterministic.
         self.logger.debug("_update_vif_port: %s %s", port, add)
         iface_id = port.ext_ids.get('iface-id')
         if iface_id is None:
@@ -191,7 +208,11 @@ class OVSSwitch(object):
             return
 
         if not add:
-            self.network_api.remove_port(network_id, self.dpid, port.ofport)
+            try:
+                self.network_api.remove_port(network_id,
+                                             self.dpid, port.ofport)
+            except (network.NetworkNotFound, ryu_exc.PortNotFound) as e:
+                self.logger.debug('remove_port %s', traceback.format_exc())
             ports = self.ifaces.get_key(iface_id, QuantumIfaces.KEY_PORTS)
             other_ovs_ports = None
             for p in ports:
@@ -216,19 +237,23 @@ class OVSSwitch(object):
 
             try:
                 self.q_api.update_port(port.ext_ids['iface-id'], body)
-            except (q_exc.ConnectionFailed, q_exc.QuantumClientException) as e:
+            except (q_exc.ConnectionFailed, client_exc) as e:
                 self.logger.error("quantum update port failed: %s", e)
                 # TODO: When authentication failure occurred,
                 # it should get auth token again
             return
 
         # update {network, port, mac}
-        self.network_api.update_network(network_id)
-        self.network_api.update_port(network_id, self.dpid, port.ofport)
-        mac = port.ext_ids.get('attached-mac')
-        if mac:
-            self.network_api.update_mac(network_id, self.dpid, port.ofport,
-                                        mac_lib.haddr_to_bin(mac))
+        try:
+            self.network_api.update_network(network_id)
+            self.network_api.update_port(network_id, self.dpid, port.ofport)
+            mac = port.ext_ids.get('attached-mac')
+            if mac:
+                self.network_api.update_mac(network_id, self.dpid, port.ofport,
+                                            mac_lib.haddr_to_bin(mac))
+        except (network.NetworkNotFound, ryu_exc.PortNotFound) as e:
+            self.logger.debug('update network/port/mac %s',
+                              traceback.format_exc())
 
     def update_port(self, port_no, port_name, add):
         self.logger.debug('update_port port_no %d %s %s', port_no, port_name,
@@ -324,7 +349,8 @@ class QuantumAdapter(app_manager.RyuApp):
         ovs_switch = self.dps.get(dpid)
         if not ovs_switch:
             if create:
-                ovs_switch = OVSSwitch(dpid, self.nw, self.ifaces, self.logger)
+                ovs_switch = OVSSwitch(self.CONF, dpid, self.nw, self.ifaces,
+                                       self.logger)
                 self.dps[dpid] = ovs_switch
         else:
             self.logger.debug('ovs switch %s is already known', dpid)
