@@ -33,6 +33,7 @@ from ryu.exception import RyuException
 from ryu.lib import dpid as dpid_lib
 from ryu.lib import hub
 from ryu.lib import mac as mac_lib
+from ryu.lib import addrconv
 from ryu.lib.packet import arp
 from ryu.lib.packet import ethernet
 from ryu.lib.packet import icmp
@@ -45,6 +46,7 @@ from ryu.ofproto import ether
 from ryu.ofproto import inet
 from ryu.ofproto import ofproto_v1_0
 from ryu.ofproto import ofproto_v1_2
+from ryu.ofproto import ofproto_v1_3
 
 
 #=============================
@@ -205,7 +207,8 @@ class CommandFailure(RyuException):
 class RestRouterAPI(app_manager.RyuApp):
 
     OFP_VERSIONS = [ofproto_v1_0.OFP_VERSION,
-                    ofproto_v1_2.OFP_VERSION]
+                    ofproto_v1_2.OFP_VERSION,
+                    ofproto_v1_3.OFP_VERSION]
 
     _CONTEXTS = {'dpset': dpset.DPSet,
                  'wsgi': WSGIApplication}
@@ -275,7 +278,11 @@ class RestRouterAPI(app_manager.RyuApp):
         event, msgs = self.waiters[dp.id][msg.xid]
         msgs.append(msg)
 
-        if msg.flags & dp.ofproto.OFPSF_REPLY_MORE:
+        if ofproto_v1_3.OFP_VERSION == dp.ofproto.OFP_VERSION:
+            more = dp.ofproto.OFPMPF_REPLY_MORE
+        else:
+            more = dp.ofproto.OFPSF_REPLY_MORE
+        if msg.flags & more:
             return
         del self.waiters[dp.id][msg.xid]
         event.set()
@@ -285,7 +292,7 @@ class RestRouterAPI(app_manager.RyuApp):
     def stats_reply_handler_v1_0(self, ev):
         self._stats_reply_handler(ev)
 
-    # for OpenFlow version1.2
+    # for OpenFlow version1.2/1.3
     @set_ev_cls(ofp_event.EventOFPStatsReply, MAIN_DISPATCHER)
     def stats_reply_handler_v1_2(self, ev):
         self._stats_reply_handler(ev)
@@ -440,7 +447,7 @@ class Router(dict):
         ofctl = OfCtl.factory(dp, logger)
         cookie = COOKIE_DEFAULT_ID
 
-        # Set SW config: TTL error packet in (only OFPv1.2)
+        # Set SW config: TTL error packet in (for OFPv1.2/1.3)
         ofctl.set_sw_config_for_ttl()
 
         # Set flow: ARP handling (packet in)
@@ -924,9 +931,10 @@ class VlanRouter(object):
         return relate_list
 
     def packet_in_handler(self, msg, header_list):
-        # Check invalid TTL (only OpenFlow V1.2)
+        # Check invalid TTL (for OpenFlow V1.2/1.3)
         ofproto = self.dp.ofproto
-        if ofproto.OFP_VERSION == ofproto_v1_2.OFP_VERSION:
+        if ofproto.OFP_VERSION == ofproto_v1_2.OFP_VERSION or \
+                ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
             if msg.reason == ofproto.OFPR_INVALID_TTL:
                 self._packetin_invalid_ttl(msg, header_list)
                 return
@@ -1121,8 +1129,8 @@ class VlanRouter(object):
         for send_port in self.port_data.values():
             if in_port is None or in_port != send_port.port_no:
                 src_mac = send_port.mac
-                dst_mac = mac_lib.BROADCAST
-                arp_target_mac = mac_lib.DONTCARE
+                dst_mac = mac_lib.BROADCAST_STR
+                arp_target_mac = mac_lib.DONTCARE_STR
                 inport = self.ofctl.dp.ofproto.OFPP_CONTROLLER
                 output = send_port.port_no
                 self.ofctl.send_arp(arp.ARP_REQUEST, self.vlan_id,
@@ -1457,7 +1465,7 @@ class OfCtl(object):
         self.logger = logger
 
     def set_sw_config_for_ttl(self):
-        # OpenFlow v1_2 only.
+        # OpenFlow v1_2/1_3.
         pass
 
     def set_flow(self, cookie, priority, dl_type=0, dl_dst=0, dl_vlan=0,
@@ -1627,18 +1635,18 @@ class OfCtl_v1_0(OfCtl):
             v = (32 - src_mask) << ofp.OFPFW_NW_SRC_SHIFT | \
                 ~ofp.OFPFW_NW_SRC_MASK
             wildcards &= v
+            nw_src = ipv4_text_to_int(nw_src)
         if nw_dst:
             v = (32 - dst_mask) << ofp.OFPFW_NW_DST_SHIFT | \
                 ~ofp.OFPFW_NW_DST_MASK
             wildcards &= v
+            nw_dst = ipv4_text_to_int(nw_dst)
         if nw_proto:
             wildcards &= ~ofp.OFPFW_NW_PROTO
 
         match = ofp_parser.OFPMatch(wildcards, 0, 0, dl_dst, dl_vlan, 0,
                                     dl_type, 0, nw_proto,
-                                    ipv4_bytes_to_int(nw_src),
-                                    ipv4_bytes_to_int(nw_dst),
-                                    0, 0)
+                                    nw_src, nw_dst, 0, 0)
         actions = actions or []
 
         m = ofp_parser.OFPFlowMod(self.dp, match, cookie, cmd,
@@ -1656,9 +1664,11 @@ class OfCtl_v1_0(OfCtl):
         # Decrement TTL value is not supported at OpenFlow V1.0
         actions = []
         if src_mac:
-            actions.append(ofp_parser.OFPActionSetDlSrc(src_mac))
+            actions.append(ofp_parser.OFPActionSetDlSrc(
+                           mac_lib.haddr_to_bin(src_mac)))
         if dst_mac:
-            actions.append(ofp_parser.OFPActionSetDlDst(dst_mac))
+            actions.append(ofp_parser.OFPActionSetDlDst(
+                           mac_lib.haddr_to_bin(dst_mac)))
         if outport is not None:
             actions.append(ofp_parser.OFPActionOutput(outport))
 
@@ -1680,20 +1690,13 @@ class OfCtl_v1_0(OfCtl):
         self.logger.info('Delete flow [cookie=0x%x]', cookie, extra=self.sw_id)
 
 
-@OfCtl.register_of_version(ofproto_v1_2.OFP_VERSION)
-class OfCtl_v1_2(OfCtl):
+class OfCtl_after_v1_2(OfCtl):
 
     def __init__(self, dp, logger):
-        super(OfCtl_v1_2, self).__init__(dp, logger)
+        super(OfCtl_after_v1_2, self).__init__(dp, logger)
 
     def set_sw_config_for_ttl(self):
-        flags = self.dp.ofproto.OFPC_INVALID_TTL_TO_CONTROLLER
-        miss_send_len = UINT16_MAX
-        m = self.dp.ofproto_parser.OFPSetConfig(self.dp, flags,
-                                                miss_send_len)
-        self.dp.send_msg(m)
-        self.logger.info('Set SW config for TTL error packet in.',
-                         extra=self.sw_id)
+        pass
 
     def get_packetin_inport(self, msg):
         in_port = self.dp.ofproto.OFPP_ANY
@@ -1704,13 +1707,7 @@ class OfCtl_v1_2(OfCtl):
         return in_port
 
     def get_all_flow(self, waiters):
-        ofp = self.dp.ofproto
-        ofp_parser = self.dp.ofproto_parser
-
-        match = ofp_parser.OFPMatch()
-        stats = ofp_parser.OFPFlowStatsRequest(self.dp, 0, ofp.OFPP_ANY,
-                                               ofp.OFPG_ANY, 0, 0, match)
-        return self.send_stats_request(stats, waiters)
+        pass
 
     def set_flow(self, cookie, priority, dl_type=0, dl_dst=0, dl_vlan=0,
                  nw_src=0, src_mask=32, nw_dst=0, dst_mask=32,
@@ -1728,10 +1725,10 @@ class OfCtl_v1_2(OfCtl):
         if dl_vlan:
             match.set_vlan_vid(dl_vlan)
         if nw_src:
-            match.set_ipv4_src_masked(ipv4_bytes_to_int(nw_src),
+            match.set_ipv4_src_masked(ipv4_text_to_int(nw_src),
                                       mask_ntob(src_mask))
         if nw_dst:
-            match.set_ipv4_dst_masked(ipv4_bytes_to_int(nw_dst),
+            match.set_ipv4_dst_masked(ipv4_text_to_int(nw_dst),
                                       mask_ntob(dst_mask))
         if nw_proto:
             if dl_type == ether.ETH_TYPE_IP:
@@ -1761,13 +1758,9 @@ class OfCtl_v1_2(OfCtl):
         if dec_ttl:
             actions.append(ofp_parser.OFPActionDecNwTtl())
         if src_mac:
-            set_src = ofp_parser.OFPMatchField.make(ofp.OXM_OF_ETH_SRC,
-                                                    src_mac)
-            actions.append(ofp_parser.OFPActionSetField(set_src))
+            actions.append(ofp_parser.OFPActionSetField(eth_src=src_mac))
         if dst_mac:
-            set_dst = ofp_parser.OFPMatchField.make(ofp.OXM_OF_ETH_DST,
-                                                    dst_mac)
-            actions.append(ofp_parser.OFPActionSetField(set_dst))
+            actions.append(ofp_parser.OFPActionSetField(eth_dst=dst_mac))
         if outport is not None:
             actions.append(ofp_parser.OFPActionOutput(outport, 0))
 
@@ -1793,9 +1786,66 @@ class OfCtl_v1_2(OfCtl):
         self.logger.info('Delete flow [cookie=0x%x]', cookie, extra=self.sw_id)
 
 
+@OfCtl.register_of_version(ofproto_v1_2.OFP_VERSION)
+class OfCtl_v1_2(OfCtl_after_v1_2):
+
+    def __init__(self, dp, logger):
+        super(OfCtl_v1_2, self).__init__(dp, logger)
+
+    def set_sw_config_for_ttl(self):
+        flags = self.dp.ofproto.OFPC_INVALID_TTL_TO_CONTROLLER
+        miss_send_len = UINT16_MAX
+        m = self.dp.ofproto_parser.OFPSetConfig(self.dp, flags,
+                                                miss_send_len)
+        self.dp.send_msg(m)
+        self.logger.info('Set SW config for TTL error packet in.',
+                         extra=self.sw_id)
+
+    def get_all_flow(self, waiters):
+        ofp = self.dp.ofproto
+        ofp_parser = self.dp.ofproto_parser
+
+        match = ofp_parser.OFPMatch()
+        stats = ofp_parser.OFPFlowStatsRequest(self.dp, 0, ofp.OFPP_ANY,
+                                               ofp.OFPG_ANY, 0, 0, match)
+        return self.send_stats_request(stats, waiters)
+
+
+@OfCtl.register_of_version(ofproto_v1_3.OFP_VERSION)
+class OfCtl_v1_3(OfCtl_after_v1_2):
+
+    def __init__(self, dp, logger):
+        super(OfCtl_v1_3, self).__init__(dp, logger)
+
+    def set_sw_config_for_ttl(self):
+        packet_in_mask = (1 << self.dp.ofproto.OFPR_ACTION |
+                          1 << self.dp.ofproto.OFPR_INVALID_TTL)
+        port_status_mask = (1 << self.dp.ofproto.OFPPR_ADD |
+                            1 << self.dp.ofproto.OFPPR_DELETE |
+                            1 << self.dp.ofproto.OFPPR_MODIFY)
+        flow_removed_mask = (1 << self.dp.ofproto.OFPRR_IDLE_TIMEOUT |
+                             1 << self.dp.ofproto.OFPRR_HARD_TIMEOUT |
+                             1 << self.dp.ofproto.OFPRR_DELETE)
+        m = self.dp.ofproto_parser.OFPSetAsync(
+            self.dp, [packet_in_mask, 0], [port_status_mask, 0],
+            [flow_removed_mask, 0])
+        self.dp.send_msg(m)
+        self.logger.info('Set SW config for TTL error packet in.',
+                         extra=self.sw_id)
+
+    def get_all_flow(self, waiters):
+        ofp = self.dp.ofproto
+        ofp_parser = self.dp.ofproto_parser
+
+        match = ofp_parser.OFPMatch()
+        stats = ofp_parser.OFPFlowStatsRequest(self.dp, 0, 0, ofp.OFPP_ANY,
+                                               ofp.OFPG_ANY, 0, 0, match)
+        return self.send_stats_request(stats, waiters)
+
+
 def ip_addr_aton(ip_str, err_msg=None):
     try:
-        return socket.inet_aton(ip_str)
+        return addrconv.ipv4.bin_to_text(socket.inet_aton(ip_str))
     except (struct.error, socket.error) as e:
         if err_msg is not None:
             e.message = '%s %s' % (err_msg, e.message)
@@ -1803,7 +1853,7 @@ def ip_addr_aton(ip_str, err_msg=None):
 
 
 def ip_addr_ntoa(ip):
-    return socket.inet_ntoa(ip)
+    return socket.inet_ntoa(addrconv.ipv4.text_to_bin(ip))
 
 
 def mask_ntob(mask, err_msg=None):
@@ -1819,21 +1869,21 @@ def mask_ntob(mask, err_msg=None):
 def ipv4_apply_mask(address, prefix_len, err_msg=None):
     import itertools
 
-    assert isinstance(address, bytes)
-    assert len(address) == 4
-    mask = ipv4_int_to_bytes(mask_ntob(prefix_len, err_msg))
-    return ''.join(chr(ord(x) & ord(y)) for (x, y) in
-                   itertools.izip(address, mask))
+    assert isinstance(address, str)
+    address_int = ipv4_text_to_int(address)
+    return ipv4_int_to_text(address_int & mask_ntob(prefix_len, err_msg))
 
 
-def ipv4_int_to_bytes(ip_int):
+def ipv4_int_to_text(ip_int):
     assert isinstance(ip_int, (int, long))
-    return struct.pack('!I', ip_int)
+    return addrconv.ipv4.bin_to_text(struct.pack('!I', ip_int))
 
 
-def ipv4_bytes_to_int(ip_bytes):
-    assert isinstance(ip_bytes, bytes)
-    return struct.unpack('!I', ip_bytes)[0]
+def ipv4_text_to_int(ip_text):
+    if ip_text == 0:
+        return ip_text
+    assert isinstance(ip_text, str)
+    return struct.unpack('!I', addrconv.ipv4.text_to_bin(ip_text))[0]
 
 
 def nw_addr_aton(nw_addr, err_msg=None):
