@@ -25,7 +25,7 @@
 
 import ryu.contrib
 
-from oslo.config import cfg
+from ryu import cfg
 
 import cmd
 import sys
