@@ -369,21 +369,36 @@ def to_match_tpdst(value, match, rest):
 
 def to_match_ip(value):
     ip_mask = value.split('/')
-    # ip
+
+    # IP address
     ipv4 = struct.unpack('!I', socket.inet_aton(ip_mask[0]))[0]
     # netmask
-    mask = 32
+    netmask = ofproto_v1_3_parser.UINT32_MAX
+
     if len(ip_mask) == 2:
-        mask = int(ip_mask[1])
-    netmask = ofproto_v1_3_parser.UINT32_MAX << 32 - mask\
-        & ofproto_v1_3_parser.UINT32_MAX
+        # Check the mask is CIDR or not.
+        if ip_mask[1].isdigit():
+            netmask &= ofproto_v1_3_parser.UINT32_MAX << 32 - int(ip_mask[1])
+        else:
+            netmask = struct.unpack('!I', socket.inet_aton(ip_mask[1]))[0]
 
     return ipv4, netmask
 
 
 def to_match_ipv6(value):
-    ip = netaddr.IPNetwork(value)
-    return ip.ip.words, ip.netmask.words
+    ip_mask = value.split('/')
+
+    if len(ip_mask) == 2 and ip_mask[1].isdigit() is False:
+        # Both address and netmask are colon-hexadecimal.
+        ipv6 = netaddr.IPAddress(ip_mask[0]).words
+        netmask = netaddr.IPAddress(ip_mask[1]).words
+    else:
+        # For other formats.
+        network = netaddr.IPNetwork(value)
+        ipv6 = network.ip.words
+        netmask = network.netmask.words
+
+    return ipv6, netmask
 
 
 def to_match_metadata(value):
@@ -469,8 +484,11 @@ def match_ip_to_str(value, mask):
     ip = socket.inet_ntoa(struct.pack('!I', value))
 
     if mask is not None and mask != 0:
-        binary_str = bin(mask)[2:].zfill(8)
-        netmask = '/%d' % len(binary_str.rstrip('0'))
+        binary_str = bin(mask)[2:].zfill(32).rstrip('0')
+        if binary_str.find('0') >= 0:
+            netmask = '/%s' % socket.inet_ntoa(struct.pack('!I', mask))
+        else:
+            netmask = '/%d' % len(binary_str)
     else:
         netmask = ''
 
@@ -484,14 +502,21 @@ def match_ipv6_to_str(value, mask):
     ip = netaddr.IPNetwork(':'.join(ip_list))
 
     netmask = 128
+    netmask_str = None
     if mask is not None:
         mask_list = []
         for word in mask:
             mask_list.append('%04x' % word)
         mask_v = netaddr.IPNetwork(':'.join(mask_list))
-        netmask = len(mask_v.ip.bits().replace(':', '').rstrip('0'))
+        binary_str = mask_v.ip.bits().replace(':', '').zfill(128).rstrip('0')
+        if binary_str.find('0') >= 0:
+            netmask_str = str(mask_v.ip)
+        else:
+            netmask = len(binary_str)
 
-    if netmask == 128:
+    if netmask_str is not None:
+        ip_str = str(ip.ip) + '/' + netmask_str
+    elif netmask == 128:
         ip_str = str(ip.ip)
     else:
         ip.prefixlen = netmask
@@ -674,7 +699,11 @@ def get_meter_config(dp, waiters):
                 elif band.type == dp.ofproto.OFPMBT_EXPERIMENTER:
                     b['experimenter'] = band.experimenter
                 bands.append(b)
-            c = {'flags': flags.get(config.flags, 0),
+            c_flags = []
+            for k, v in flags.items():
+                if k & config.flags:
+                    c_flags.append(v)
+            c = {'flags': c_flags,
                  'meter_id': config.meter_id,
                  'bands': bands}
             configs.append(c)
@@ -831,7 +860,12 @@ def mod_meter_entry(dp, flow, cmd):
                      'BURST': dp.ofproto.OFPMF_BURST,
                      'STATS': dp.ofproto.OFPMF_STATS}
 
-    flags = flags_convert.get(flow.get('flags'))
+    flow_flags = flow.get('flags')
+    if not isinstance(flow_flags, list):
+        flow_flags = [flow_flags]
+    flags = 0
+    for flag in flow_flags:
+        flags |= flags_convert.get(flag, 0)
     if not flags:
         LOG.debug('Unknown flags: %s', flow.get('flags'))
 
@@ -872,7 +906,7 @@ def mod_group_entry(dp, group, cmd):
                     'FF': dp.ofproto.OFPGT_FF}
 
     type_ = type_convert.get(group.get('type'))
-    if not type_:
+    if type_ is None:
         LOG.debug('Unknown type: %s', group.get('type'))
 
     group_id = int(group.get('group_id', 0))
