@@ -1259,7 +1259,12 @@ OFP_QUEUE_STATS_SIZE = 48
 assert calcsize(OFP_QUEUE_STATS_PACK_STR) == OFP_QUEUE_STATS_SIZE
 
 # struct ofp_flow_monitor_request
-OFP_FLOW_MONITOR_REQUEST_PACK_STR = '!IIIHBB' + _OFP_MATCH_PACK_STR
+_OFP_FLOW_MONITOR_REQUEST_0_PACK_STR = 'IIIHBB'
+OFP_FLOW_MONITOR_REQUEST_0_PACK_STR = ('!' +
+                                       _OFP_FLOW_MONITOR_REQUEST_0_PACK_STR)
+OFP_FLOW_MONITOR_REQUEST_0_SIZE = 16
+OFP_FLOW_MONITOR_REQUEST_PACK_STR = (OFP_FLOW_MONITOR_REQUEST_0_PACK_STR +
+                                     _OFP_MATCH_PACK_STR)
 OFP_FLOW_MONITOR_REQUEST_SIZE = 24
 assert (calcsize(OFP_FLOW_MONITOR_REQUEST_PACK_STR) ==
         OFP_FLOW_MONITOR_REQUEST_SIZE)
@@ -1299,7 +1304,11 @@ OFPFME_PAUSED = 5  # Monitoring paused (out of buffer space).
 OFPFME_RESUMED = 6  # Monitoring resumed.
 
 # struct ofp_flow_update_full
-OFP_FLOW_UPDATE_FULL_PACK_STR = '!HHBBHHH4xQ' + _OFP_MATCH_PACK_STR
+_OFP_FLOW_UPDATE_FULL_0_PACK_STR = 'HHBBHHH4xQ'
+OFP_FLOW_UPDATE_FULL_0_PACK_STR = '!' + _OFP_FLOW_UPDATE_FULL_0_PACK_STR
+OFP_FLOW_UPDATE_FULL_0_SIZE = 24
+OFP_FLOW_UPDATE_FULL_PACK_STR = (OFP_FLOW_UPDATE_FULL_0_PACK_STR +
+                                 _OFP_MATCH_PACK_STR)
 OFP_FLOW_UPDATE_FULL_SIZE = 32
 assert (calcsize(OFP_FLOW_UPDATE_FULL_PACK_STR) ==
         OFP_FLOW_UPDATE_FULL_SIZE)
@@ -1366,12 +1375,6 @@ OFP_ROLE_STATUS_SIZE = 24
 assert (calcsize(OFP_ROLE_STATUS_PACK_STR) + OFP_HEADER_SIZE ==
         OFP_ROLE_STATUS_SIZE)
 
-# struct ofp_async_config
-OFP_ASYNC_CONFIG_PACK_STR = '!2I2I2I'
-OFP_ASYNC_CONFIG_SIZE = 32
-assert (calcsize(OFP_ASYNC_CONFIG_PACK_STR) + OFP_HEADER_SIZE ==
-        OFP_ASYNC_CONFIG_SIZE)
-
 # enum ofp_async_config_prop_type
 OFPACPT_PACKET_IN_SLAVE = 0  # Packet-in mask for slave.
 OFPACPT_PACKET_IN_MASTER = 1  # Packet-in mask for master.
@@ -1405,7 +1408,11 @@ OFPTR_VACANCY_DOWN = 3  # Vacancy down threshold event.
 OFPTR_VACANCY_UP = 4  # Vacancy up threshold event.
 
 # struct ofp_table_status
-OFP_TABLE_STATUS_PACK_STR = '!B7x' + _OFP_TABLE_DESC_PACK_STR
+_OFP_TABLE_STATUS_0_PACK_STR = 'B7x'
+OFP_TABLE_STATUS_0_PACK_STR = '!' + _OFP_TABLE_STATUS_0_PACK_STR
+OFP_TABLE_STATUS_0_SIZE = 16
+OFP_TABLE_STATUS_PACK_STR = (OFP_TABLE_STATUS_0_PACK_STR +
+                             _OFP_TABLE_DESC_PACK_STR)
 OFP_TABLE_STATUS_SIZE = 24
 assert (calcsize(OFP_TABLE_STATUS_PACK_STR) + OFP_HEADER_SIZE ==
         OFP_TABLE_STATUS_SIZE)
@@ -1444,7 +1451,10 @@ assert (calcsize(OFP_BUNDLE_CTRL_MSG_PACK_STR) + OFP_HEADER_SIZE ==
         OFP_BUNDLE_CTRL_MSG_SIZE)
 
 # struct ofp_bundle_add_msg
-OFP_BUNDLE_ADD_MSG_PACK_STR = '!IHH' + _OFP_HEADER_PACK_STR
+_OFP_BUNDLE_ADD_MSG_0_PACK_STR = 'I2xH'
+OFP_BUNDLE_ADD_MSG_0_PACK_STR = '!' + _OFP_BUNDLE_ADD_MSG_0_PACK_STR
+OFP_BUNDLE_ADD_MSG_PACK_STR = (OFP_BUNDLE_ADD_MSG_0_PACK_STR +
+                               _OFP_HEADER_PACK_STR)
 OFP_BUNDLE_ADD_MSG_SIZE = 24
 assert (calcsize(OFP_BUNDLE_ADD_MSG_PACK_STR) + OFP_HEADER_SIZE ==
         OFP_BUNDLE_ADD_MSG_SIZE)
